@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright 2019-2020 Pelion
+# Copyright 2019-2021 Pelion
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -22,10 +22,11 @@ from pathlib import Path
 from typing import Type
 
 import yaml
-from requests import HTTPError
+import requests
 
 from manifesttool.dev_tool import defaults
-from manifesttool.dev_tool.actions.create import bump_minor
+from manifesttool.dev_tool.actions.create \
+    import load_cfg_and_get_fw_ver
 from manifesttool.dev_tool.actions.create \
     import register_parser as register_create_parser
 from manifesttool.dev_tool.actions.create import create_dev_manifest
@@ -69,7 +70,6 @@ def register_parser(parser: argparse.ArgumentParser, schema_version: str):
         '-t', '--timeout',
         type=non_negative_int_arg_factory,
         help='Wait timeout in seconds. '
-             'Only relevant if --wait-for-completion was provided. '
              '[Default: 0 - wait-forever]',
         default=0
     )
@@ -80,31 +80,54 @@ def register_parser(parser: argparse.ArgumentParser, schema_version: str):
              'Only relevant if --wait-for-completion was provided.'
     )
 
-
-def _wait(api: pelion.UpdateServiceApi, campaign_id: pelion.ID, timeout):
+def _manage_campaign(api: pelion.UpdateServiceApi,
+                     campaign_id: pelion.ID,
+                     end_time: int,
+                     do_wait: bool):
+    starting = True
+    old_state = ''
     try:
-        old_state = api.campaign_get(campaign_id)['state']
-        logger.info("Campaign state: %s", old_state)
-        current_time = time.time()
-        while timeout == 0 or time.time() < current_time + timeout:
-            curr_campaign = api.campaign_get(campaign_id)
+        api.campaign_start(campaign_id)
+    except requests.HTTPError as ex:
+        logger.error('Failed to start campaign %s', campaign_id)
+        raise
+    try:
+        while end_time == 0 or time.time() < end_time:
+            try:
+                curr_campaign = api.campaign_get(campaign_id)
+            except requests.ConnectionError as ex:
+                logger.debug('Connection error %s', ex)
+                time.sleep(3)
+                continue
             curr_state = curr_campaign['state']
             curr_phase = curr_campaign['phase']
+            if starting:
+                if api.campaign_is_not_started(curr_phase):
+                    raise AssertionError(
+                        'Campaign not started - check filter and campaign '
+                        'state.\n'
+                        'Reason: {}'.format(curr_campaign['autostop_reason']))
+                if api.campaign_is_active(curr_phase):
+                    logger.info('Started Campaign ID: %s', campaign_id)
+                    starting = False
+                    if not do_wait:
+                        return
+            else:
+                if not api.campaign_is_active(curr_phase):
+                    logger.info(
+                        "Campaign is finished in state: %s", curr_state)
+                    return
             if old_state != curr_state:
                 logger.info("Campaign state: %s", curr_state)
                 old_state = curr_state
-            if not api.campaign_is_active(curr_phase):
-                logger.info(
-                    "Campaign is finished in state: %s", curr_state)
-                return
-            time.sleep(1)
+            time.sleep(3)
         logger.error('Campaign timed out')
         raise AssertionError('Campaign timed out')
-    except HTTPError:
+    except requests.HTTPError as ex:
         logger.error('Failed to retrieve campaign state')
         raise
 
-def _print_summary(summary: dict):
+def _print_summary(summary: dict, summary_reasons: dict):
     logger.info("----------------------------")
     logger.info("    Campaign Summary ")
     logger.info("----------------------------")
@@ -117,6 +140,16 @@ def _print_summary(summary: dict):
     logger.info(" Pending:                %d",
                 summary.get('info') if summary.get('info') else 0)
     logger.info(" Total in this campaign: %d", sum(summary.values()))
+    if summary_reasons.get('fail'):
+        logger.warning("Reasons for failed updates:")
+        for reasons in summary_reasons['fail']:
+            if reasons.get('description'):
+                logger.warning(" %s", reasons.get('description'))
+    if summary_reasons.get('skipped'):
+        logger.warning("Reasons for skipped updates:")
+        for reasons in summary_reasons['skipped']:
+            if reasons.get('description'):
+                logger.warning(" %s", reasons.get('description'))
 
 
 def _finalize(api: pelion.UpdateServiceApi,
@@ -125,19 +158,25 @@ def _finalize(api: pelion.UpdateServiceApi,
               manifest_id: pelion.ID,
               fw_image_id: pelion.ID,
               manifest_path: Path):
-    summary = []
+    summary = {}
+    summary_reasons = {}
     failed_devices = []
     try:
         if campaign_id:
             # stop the campaign if it's still active
             api.campaign_stop(campaign_id)
-            # get summary and failed devices
+            # get summary, failed devices and reasons
             statistics = api.campaign_statistics(campaign_id)
             summary = {s['id']: s['count'] for s in statistics}
             if summary.get('fail') and summary.get('fail') > 0:
                 devices_state = api.campaign_device_metadata(campaign_id)
                 failed_devices = [d['device_id'] for d in devices_state
                                   if d['deployment_state'] != 'deployed']
+                summary_reasons['fail'] = \
+                    api.campaign_statistic_events(campaign_id, 'fail')
+            if summary.get('skipped') and summary.get('skipped') > 0:
+                summary_reasons['skipped'] = \
+                    api.campaign_statistic_events(campaign_id, 'skipped')
         if do_cleanup:
             logger.info('Cleaning up resources...')
             if campaign_id:
@@ -148,13 +187,13 @@ def _finalize(api: pelion.UpdateServiceApi,
                 api.fw_delete(fw_image_id)
             if manifest_path and manifest_path.is_file():
                 manifest_path.unlink()
-    except HTTPError as ex:
+    except requests.HTTPError as ex:
         logger.error('Failed to finalize update campaign')
         logger.debug('Exception %s', ex, exc_info=True)
 
     # print campaign summary
     if summary:
-        _print_summary(summary)
+        _print_summary(summary, summary_reasons)
         if failed_devices:
             # assert if not all devices were updated
             raise AssertionError(
@@ -171,7 +210,7 @@ def update(
         device_id: str,
         do_wait: bool,
         do_start: bool,
-        timeout: int,
+        end_time: int,
         do_cleanup: bool,
         service_config: Path,
         fw_version: str,
@@ -182,7 +221,7 @@ def update(
     config = load_service_config(service_config)
 
     api = pelion.UpdateServiceApi(
-        host=config['host'], api_key=config['api_key'])
+        host=config['host'], access_key=config['access_key'])
     manifest_path = None
     fw_image_id = None
     manifest_id = None
@@ -239,10 +278,7 @@ def update(
         )
 
         if do_start:
-            api.campaign_start(campaign_id)
-
-        if do_wait:
-            _wait(api, campaign_id, timeout)
+            _manage_campaign(api, campaign_id, end_time, do_wait)
 
     except KeyboardInterrupt:
         logger.error('Aborted by user...')
@@ -260,7 +296,11 @@ def load_service_config(service_config):
     if service_config.is_file():
         with service_config.open('rt') as fh:
             config = yaml.safe_load(fh)
+            if 'host' in config and 'access_key' in config:
+                return config
             if 'host' in config and 'api_key' in config:
+                # replace api_key with access_key for backward compatibility
+                config['access_key'] = config.pop('api_key')
                 return config
     raise AssertionError('Pelion service configurations (URL and access key) '
                          'are not provided for assisted campaign '
@@ -271,36 +311,8 @@ def entry_point(
         args,
         manifest_version: Type[ManifestAsnCodecBase]
 ):
-
-    cache_dir = args.cache_dir
-
-    if not cache_dir.is_dir():
-        raise AssertionError('Tool cache directory is missing. '
-                             'Execute "init" command to create it.')
-
-    with (cache_dir / defaults.DEV_CFG).open('rt') as fh:
-        dev_cfg = yaml.safe_load(fh)
-    component_name = getattr(args, 'component_name', None)
-    cache_fw_version_file = cache_dir / defaults.UPDATE_VERSION
-    fw_version = args.fw_version
-    if 'v1' not in manifest_version.get_name():
-        if cache_fw_version_file.is_file():
-            with cache_fw_version_file.open('rt') as fh:
-                cached_versions = yaml.safe_load(fh)
-        else:
-            cached_versions = dict()
-        if not fw_version:
-            fw_version = cached_versions.get(component_name, '0.0.1')
-            fw_version = bump_minor(fw_version)
-
-        cached_versions[component_name] = fw_version
-
-        with cache_fw_version_file.open('wt') as fh:
-            yaml.dump(cached_versions, fh)
-        logger.info('FW version: %s', fw_version)
-    else:
-        assert fw_version is not None
-        logger.info('FW version: %d', fw_version)
+    dev_cfg, fw_version = \
+        load_cfg_and_get_fw_ver(args, manifest_version)
 
     update(
         payload_path=args.payload_path,
@@ -311,11 +323,11 @@ def entry_point(
         device_id=args.device_id,
         do_start=args.start_campaign or args.wait,
         do_wait=args.wait,
-        timeout=args.timeout,
+        end_time=time.time() + args.timeout if args.timeout > 0 else 0,
         do_cleanup=not args.no_cleanup,
-        service_config=cache_dir / defaults.CLOUD_CFG,
+        service_config=args.cache_dir / defaults.CLOUD_CFG,
         fw_version=fw_version,
         sign_image=getattr(args, 'sign_image', False),
-        component=component_name,
+        component=getattr(args, 'component_name', None),
         short_url=hasattr(args, 'use_short_url') and args.use_short_url
     )
