@@ -1,5 +1,5 @@
 # ----------------------------------------------------------------------------
-# Copyright 2019-2020 Pelion
+# Copyright 2019-2021 Pelion
 #
 # SPDX-License-Identifier: Apache-2.0
 #
@@ -39,6 +39,8 @@ FW_CAMPAIGN = '/v3/update-campaigns/{id}'
 FW_CAMPAIGN_STOP = '/v3/update-campaigns/{id}/stop'
 FW_CAMPAIGN_START = '/v3/update-campaigns/{id}/start'
 FW_CAMPAIGN_STATISTICS = '/v3/update-campaigns/{id}/statistics'
+FW_CAMPAIGN_STATISTICS_EVENTS = \
+    '/v3/update-campaigns/{id}/statistics/{summary_id}/event_types'
 FW_CAMPAIGN_DEV_METADATA = '/v3/update-campaigns/{id}/campaign-device-metadata'
 
 CAMPAIGN_ACTIVE_PHASES = [
@@ -46,6 +48,11 @@ CAMPAIGN_ACTIVE_PHASES = [
     'active'
 ]
 
+CAMPAIGN_NOT_STARTED_PHASES = [
+    'draft',
+    'stopped'
+]
+
 FW_UPLOAD_MAX_SMALL_SIZE = int(100 * 1024 * 1024)
 
 # must be greater than 5MB - AWS limitation
@@ -58,15 +65,15 @@ ID = NewType('ID', str)
 Phase = NewType('Phase', str)
 
 class UpdateServiceApi:
-    def __init__(self, host: URL, api_key: str):
+    def __init__(self, host: URL, access_key: str):
         """
         Create REST API provider for Update Service APIs
         :param host: Pelion service URL
-        :param api_key: account API key
+        :param access_key: account access key
         """
         self._host = host
         self._default_headers = {
-            'Authorization': 'Bearer {}'.format(api_key)
+            'Authorization': 'Bearer {}'.format(access_key)
         }
 
     def _url(self, api, **kwargs) -> str:
@@ -322,7 +329,7 @@ class UpdateServiceApi:
             device_filter: str
     ) -> ID:
         """
-        Create update campaign om Pelion portal
+        Create update campaign on Pelion portal
 
         :param name: campaign name as will appear on Pelion portal
         :param manifest_id: manifest ID as
@@ -393,32 +400,14 @@ class UpdateServiceApi:
 
     def campaign_start(self, campaign_id: ID):
         """
-        Start draft update campaign
+        Start update campaign
         :param campaign_id: campaign ID
         """
-        try:
-            response = requests.post(
-                self._url(FW_CAMPAIGN_START, id=campaign_id),
-                headers=self._headers()
-            )
-            response.raise_for_status()
-
-            time.sleep(1)
-
-            campaign = self.campaign_get(campaign_id)
-            while True:
-                if campaign['phase'] == 'draft':
-                    raise AssertionError(
-                        'Campaign not started - check filter and campaign '
-                        'state.\n'
-                        'Reason: {}'.format(campaign['autostop_reason']))
-                if campaign['phase'] == 'active':
-                    break
-                campaign = self.campaign_get(campaign_id)
-        except requests.HTTPError:
-            LOG.error('Failed to start campaign %s', campaign_id)
-            raise
-        LOG.info('Started Campaign ID: %s', campaign_id)
+        response = requests.post(
+            self._url(FW_CAMPAIGN_START, id=campaign_id),
+            headers=self._headers()
+        )
+        response.raise_for_status()
 
     def campaign_get(self, campaign_id: ID) -> dict:
         """
@@ -451,6 +440,29 @@ class UpdateServiceApi:
             pass
         return []
 
+    def campaign_statistic_events(self,
+                                  campaign_id: ID,
+                                  summary_id: str) -> List[dict]:
+        """
+        Get campaign events grouped by summary
+
+        :param campaign_id: campaign ID
+        :param summary_id: The summary status
+            Available values: fail, success, info, skipped
+        :return: List of statistics for a campaign
+        """
+        try:
+            response = requests.get(
+                self._url(FW_CAMPAIGN_STATISTICS_EVENTS,
+                          id=campaign_id, summary_id=summary_id),
+                headers=self._headers()
+            )
+            response.raise_for_status()
+            return response.json()['data']
+        except requests.HTTPError:
+            pass
+        return []
+
     def campaign_device_metadata(self, campaign_id: ID) -> List[dict]:
         """
         Get metadata for devices participating in update campaign
@@ -477,3 +489,13 @@ class UpdateServiceApi:
         :return: True if update campaign phase is starting or active
         """
         return phase in CAMPAIGN_ACTIVE_PHASES
+
+    @staticmethod
+    def campaign_is_not_started(phase: Phase) -> bool:
+        """
+        Helper function for understanding if update campaign is not
+        started phase
+        :param phase: current update campaign phase
+        :return: True if update campaign phase is starting or active
+        """
+        return phase in CAMPAIGN_NOT_STARTED_PHASES
